__author__ = 'Oleg Butovich'
__copyright__ = '(c) Oleg Butovich 2013-2015'
__licence__ = 'MIT'


import os
from proxmoxer.backends.base_ssh import ProxmoxBaseSSHSession, BaseBackend

try:
    import paramiko
except ImportError:
    import sys
    sys.stderr.write("Chosen backend requires 'paramiko' module\n")
    sys.exit(1)


class ProxmoxParamikoSession(ProxmoxBaseSSHSession):
    def __init__(self, host,
                 username,
                 password=None,
                 private_key_file=None,
                 port=22,
                 timeout=5):
        self.host = host
        self.username = username
        self.password = password
        self.private_key_file = private_key_file
        self.port = port
        self.timeout = timeout
        self.ssh_client = self._connect()

    def _connect(self):
        ssh_client = paramiko.SSHClient()
        ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

        if self.private_key_file:
            key_filename = os.path.expanduser(self.private_key_file)
        else:
            key_filename = None

        ssh_client.connect(self.host,
                           username=self.username,
                           allow_agent=(not self.password),
                           look_for_keys=True,
                           key_filename=key_filename,
                           password=self.password,
                           timeout=self.timeout,
                           port=self.port)

        return ssh_client

    def _exec(self, cmd):
        session = self.ssh_client.get_transport().open_session()
        session.exec_command(cmd)
        stdout = ''.join(session.makefile('rb', -1))
        stderr = ''.join(session.makefile_stderr('rb', -1))
        return stdout, stderr

    def upload_file_obj(self, file_obj, remote_path):
        sftp = self.ssh_client.open_sftp()
        remote_file = sftp.open(remote_path, 'wb')
        remote_file.write(file_obj.read())
        remote_file.close()


class Backend(BaseBackend):
    def __init__(self, host, user, password=None, private_key_file=None, port=22, timeout=5):
        self.session = ProxmoxParamikoSession(host, user,
                                              password=password,
                                              private_key_file=private_key_file,
                                              port=port,
                                              timeout=timeout)





#!/usr/bin/env python
import codecs
import re
import sys
import proxmoxer
import os
from setuptools import setup


if not os.path.exists('README.txt') and 'sdist' in sys.argv:
    with codecs.open('README.rst', encoding='utf8') as f:
        rst = f.read()
    code_block = '(:\n\n)?\.\. code-block::.*'
    rst = re.sub(code_block, '::', rst)
    with codecs.open('README.txt', encoding='utf8', mode='wb') as f:
        f.write(rst)


try:
    readme = 'README.txt' if os.path.exists('README.txt') else 'README.rst'
    long_description = codecs.open(readme, encoding='utf-8').read()
except:
    long_description = 'Could not read README.txt'


setup(
    name = 'proxmoxer',
    version = proxmoxer.__version__,
    description = 'Python Wrapper for the Proxmox 2.x API (HTTP and SSH)',
    author = 'Oleg Butovich',
    author_email = 'obutovich@gmail.com',
    license = "MIT",
    url = 'https://github.com/swayf/proxmoxer',
    download_url = 'http://pypi.python.org/pypi/proxmoxer',
    keywords = ['proxmox', 'api'],
    packages=['proxmoxer', 'proxmoxer.backends'],
    classifiers = [ #http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
    long_description = long_description
)



__author__ = 'Oleg Butovich'
__copyright__ = '(c) Oleg Butovich 2013-2015'
__licence__ = 'MIT'


from proxmoxer.backends.base_ssh import ProxmoxBaseSSHSession, BaseBackend

try:
    import openssh_wrapper
except ImportError:
    import sys
    sys.stderr.write("Chosen backend requires 'openssh_wrapper' module\n")
    sys.exit(1)


class ProxmoxOpenSSHSession(ProxmoxBaseSSHSession):
    def __init__(self, host,
                 username,
                 configfile=None,
                 port=22,
                 timeout=5,
                 forward_ssh_agent=False):
        self.host = host
        self.username = username
        self.configfile = configfile
        self.port = port
        self.timeout = timeout
        self.forward_ssh_agent = forward_ssh_agent
        self.ssh_client = openssh_wrapper.SSHConnection(self.host,
                                                        login=self.username,
                                                        port=self.port,
                                                        timeout=self.timeout)

    def _exec(self, cmd):
        ret = self.ssh_client.run(cmd, forward_ssh_agent=self.forward_ssh_agent)
        return ret.stdout, ret.stderr

    def upload_file_obj(self, file_obj, remote_path):
        self.ssh_client.scp((file_obj,), target=remote_path)


class Backend(BaseBackend):
    def __init__(self, host, user, configfile=None, port=22, timeout=5, forward_ssh_agent=False):
        self.session = ProxmoxOpenSSHSession(host, user,
                                             configfile=configfile,
                                             port=port,
                                             timeout=timeout,
                                             forward_ssh_agent=forward_ssh_agent)



__author__ = 'Oleg Butovich'
__copyright__ = '(c) Oleg Butovich 2013-2015'
__licence__ = 'MIT'


from itertools import chain
import json


class Response(object):
    def __init__(self, content, status_code):
        self.status_code = status_code
        self.content = content
        self.headers = {"content-type": "application/json"}


class ProxmoxBaseSSHSession(object):

    def _exec(self, cmd):
        raise NotImplementedError()

    # noinspection PyUnusedLocal
    def request(self, method, url, data=None, params=None, headers=None):
        method = method.lower()
        data = data or {}
        params = params or {}
        url = url.strip()

        cmd = {'post': 'create',
               'put': 'set'}.get(method, method)

        #for 'upload' call some workaround
        tmp_filename = ''
        if url.endswith('upload'):
            #copy file to temporary location on proxmox host
            tmp_filename, _ = self._exec(
                "python -c 'import tempfile; tf = tempfile.NamedTemporaryFile(); print tf.name'")
            self.upload_file_obj(data['filename'], tmp_filename)
            data['filename'] = data['filename'].name
            data['tmpfilename'] = tmp_filename

        translated_data = ' '.join(["-{0} {1}".format(k, v) for k, v in chain(data.items(), params.items())])
        full_cmd = 'pvesh {0}'.format(' '.join(filter(None, (cmd, url, translated_data))))

        stdout, stderr = self._exec(full_cmd)
        try:
            status_code = int(stderr.split()[0])
        except:
            status_code = 500
        return Response(stdout, status_code)

    def upload_file_obj(self, file_obj, remote_path):
        raise NotImplementedError()


class JsonSimpleSerializer(object):

    def loads(self, response):
        try:
            return json.loads(response.content)
        except ValueError:
            return response.content


class BaseBackend(object):

    def get_session(self):
        return self.session

    def get_base_url(self):
        return ''

    def get_serializer(self):
        return JsonSimpleSerializer()
